@@ -3,6 +3,7 @@
 use crate::database::{Database, HasArguments};
 use crate::encode::Encode;
 use crate::types::Type;
+use std::fmt::{self, Write};
 
 /// A tuple of arguments to be sent to the database.
 pub trait Arguments<'q>: Send + Sized + Default {
@@ -16,6 +17,10 @@ pub trait Arguments<'q>: Send + Sized + Default {
     fn add<T>(&mut self, value: T)
     where
         T: 'q + Send + Encode<'q, Self::Database> + Type<Self::Database>;
+
+    fn format_placeholder<W: Write>(&self, writer: &mut W) -> fmt::Result {
+        writer.write_str("?")
+    }
 }
 
 pub trait IntoArguments<'q, DB: HasArguments<'q>>: Sized + Send {
