@@ -39,7 +39,7 @@ pub enum Error {
     Database(#[source] Box<dyn DatabaseError>),
 
     /// Error communicating with the database backend.
-    #[error("error communicating with the server: {0}")]
+    #[error("error communicating with database: {0}")]
     Io(#[from] io::Error),
 
     /// Error occurred while attempting to establish a TLS connection.
