@@ -228,7 +228,7 @@ impl<DB: Database> PoolOptions<DB> {
     }
 }
 
-async fn init_min_connections<DB: Database>(pool: &SharedPool<DB>) -> Result<(), Error> {
+async fn init_min_connections<DB: Database>(pool: &Arc<SharedPool<DB>>) -> Result<(), Error> {
     for _ in 0..cmp::max(pool.options.min_connections, 1) {
         let deadline = Instant::now() + pool.options.connect_timeout;
         let permit = pool.semaphore.acquire(1).await;
