@@ -31,11 +31,12 @@ impl PgConnection {
             ("client_encoding", "UTF8"),
             // Sets the time zone for displaying and interpreting time stamps.
             ("TimeZone", "UTC"),
-            // Adjust postgres to return precise values for floats
-            // NOTE: This is default in postgres 12+
-            ("extra_float_digits", "3"),
         ];
 
+        if let Some(ref extra_float_digits) = options.extra_float_digits {
+            params.push(("extra_float_digits", extra_float_digits));
+        }
+
         if let Some(ref application_name) = options.application_name {
             params.push(("application_name", application_name));
         }
