@@ -2,6 +2,7 @@
 
 use crate::executor::Executor;
 
+mod advisory_lock;
 mod arguments;
 mod column;
 mod connection;
@@ -23,6 +24,7 @@ mod value;
 #[cfg(feature = "migrate")]
 mod migrate;
 
+pub use advisory_lock::{PgAdvisoryLock, PgAdvisoryLockGuard, PgAdvisoryLockKey};
 pub use arguments::{PgArgumentBuffer, PgArguments};
 pub use column::PgColumn;
 pub use connection::{PgConnection, PgConnectionInfo};
