@@ -4,7 +4,7 @@ use std::str::FromStr;
 /// Refer to [SQLite documentation] for the meaning of various synchronous settings.
 ///
 /// [SQLite documentation]: https://www.sqlite.org/pragma.html#pragma_synchronous
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum SqliteSynchronous {
     Off,
     Normal,
