@@ -56,34 +56,6 @@ where
     }
 }
 
-impl<DB> Type<DB> for Vec<JsonValue>
-where
-    Vec<Json<JsonValue>>: Type<DB>,
-    DB: Database,
-{
-    fn type_info() -> DB::TypeInfo {
-        <Vec<Json<JsonValue>> as Type<DB>>::type_info()
-    }
-
-    fn compatible(ty: &DB::TypeInfo) -> bool {
-        <Vec<Json<JsonValue>> as Type<DB>>::compatible(ty)
-    }
-}
-
-impl<DB> Type<DB> for [JsonValue]
-where
-    [Json<JsonValue>]: Type<DB>,
-    DB: Database,
-{
-    fn type_info() -> DB::TypeInfo {
-        <[Json<JsonValue>] as Type<DB>>::type_info()
-    }
-
-    fn compatible(ty: &DB::TypeInfo) -> bool {
-        <[Json<JsonValue>] as Type<DB>>::compatible(ty)
-    }
-}
-
 impl<'q, DB> Encode<'q, DB> for JsonValue
 where
     for<'a> Json<&'a Self>: Encode<'q, DB>,
